@@ -9,6 +9,12 @@
     :copyright: (c) 2013 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 
+    .. versionchanged:: 0.2
+       'LINKS' defaults to '_links'.
+       'ITEMS' defaults to '_items'.
+       'STATUS' defaults to 'status'.
+       'ISSUES' defaults to 'issues'.
+
     .. versionchanged:: 0.1.1
        'SERVER_NAME' defaults to None.
 
@@ -17,7 +23,7 @@
 
 """
 
-__version__ = '0.1.1'
+__version__ = '0.2'
 
 #DEBUG = True
 
@@ -30,8 +36,6 @@ SERVER_NAME = None
 PAGINATION = True
 PAGINATION_LIMIT = 50
 PAGINATION_DEFAULT = 25
-LAST_UPDATED = 'updated'
-DATE_CREATED = 'created'
 ID_FIELD = '_id'
 CACHE_CONTROL = 'max-age=10,must-revalidate'        # TODO confirm this value
 CACHE_EXPIRES = 10
@@ -44,6 +48,12 @@ ITEM_URL = '[a-f0-9]{24}'
 
 STATUS_OK = "OK"
 STATUS_ERR = "ERR"
+LAST_UPDATED = 'updated'
+DATE_CREATED = 'created'
+ISSUES = 'issues'
+STATUS = 'status'
+ITEMS = '_items'
+LINKS = '_links'
 
 # must be the last line (will raise W402 on pyflakes)
 from eve.flaskapp import Eve  # noqa
