@@ -42,7 +42,7 @@ class RegexConverter(BaseConverter):
 
 
 class Eve(Flask, Events):
-    """The main Eve object. On initialization it will load Eve settings, then
+    """ The main Eve object. On initialization it will load Eve settings, then
     configure and enable the API endpoints. The API is launched by executing
     the code below:::
 
@@ -60,8 +60,21 @@ class Eve(Flask, Events):
                  requests. Must be a :class: `eve.auth.BasicAuth` subclass.
     :param redis: the redis (pyredis) instance used by the Rate-Limiting
                   feature, if enabled.
+    :param url_converters: dictionary of Flask url_converters to add to
+                           supported ones (int, float, path, regex).
+    :param json_encoder: custom json encoder class. Must be a
+                         JSONEncoder subclass. You probably wnat it to be
+                         as eve.io.base.BaseJSONEncoder subclass.
     :param kwargs: optional, standard, Flask parameters.
 
+    .. versionchanged:: 0.2
+       Support for additional Flask url converters.
+       Support for optional, custom json encoder class.
+       Support for endpoint-level authenticatoin classes.
+       New method Eve.register_resource() for registering new resource after
+       initialization of Eve object. This is needed for simpler initialization
+       API of all ORM/ODM extensions.
+
     .. versionchanged:: 0.1.0
        Now supporting both "trailing slashes" and "no-trailing slashes" URLs.
 
@@ -76,44 +89,70 @@ class Eve(Flask, Events):
     .. versionchanged:: 0.0.4
        'auth' argument added to handle authentication classes
     """
+    #: Allowed methods for resource endpoints
+    supported_resource_methods = ['GET', 'POST', 'DELETE']
+
+    #: Allowed methods for item endpoints
+    supported_item_methods = ['GET', 'PATCH', 'DELETE', 'PUT']
+
     def __init__(self, import_name=__package__, settings='settings.py',
                  validator=Validator, data=Mongo, auth=None, redis=None,
-                 **kwargs):
-        """Eve main WSGI app is implemented as a Flask subclass. Since we want
+                 url_converters=None, json_encoder=None, **kwargs):
+        """ Eve main WSGI app is implemented as a Flask subclass. Since we want
         to be able to launch our API by simply invoking Flask's run() method,
         we need to enhance our super-class a little bit.
 
         The tasks we need to accomplish are:
 
             1. enbale regex routing
-            2. load and validate custom API settings,
-            3. enable API endpoints
-            4. set the validator class used to validate incoming objects
-            5. activate the chosen data layer
-            6. instance the authentication layer if needed
-            7. set the redis instance to be used by the Rate-Limiting feature
+            2. enable optional url_converters, if any
+            3. enable optional json_encoder class, if any
+            4. load and validate custom API settings
+            5. enable API endpoints
+            6. set the validator class used to validate incoming objects
+            7. activate the chosen data layer
+            8. instance the authentication layer if needed
+            9. set the redis instance to be used by the Rate-Limiting feature
+
+        .. versionchanged:: 0.2
+           Support for additional, optional Flask url_converters.
+           Support for optional, custom json encoder class.
+           Support for endpoint-level authenticatoin classes.
+           Validate and set defaults for each resource
         """
 
-        # TODO should we support standard Flask parameters as well?
         super(Eve, self).__init__(import_name, **kwargs)
-        # enable regex routing
-        self.url_map.converters['regex'] = RegexConverter
+
         self.validator = validator
         self.settings = settings
 
         self.load_config()
         self.validate_domain_struct()
-        self.set_defaults()
-        self.validate_config()
-        #self.validate_schemas()
-        self._add_url_rules()
 
         self.data = data(self)
+
+        # enable regex routing
+        self.url_map.converters['regex'] = RegexConverter
+
+        # optional url_converters and json encoder
+        if url_converters:
+            self.url_map.converters.update(url_converters)
+
+        if json_encoder:
+            self.data.json_encoder_class = json_encoder
+
         self.auth = auth() if auth else None
         self.redis = redis
 
+        # validate and set defaults for each resource
+        for resource, settings in self.config['DOMAIN'].items():
+            self._set_resource_defaults(resource, settings)
+            self._validate_resource_settings(resource, settings)
+        self._add_url_rules()
+
     def run(self, host=None, port=None, debug=None, **options):
-        """Pass our own subclass of :class:`werkzeug.serving.WSGIRequestHandler
+        """
+        Pass our own subclass of :class:`werkzeug.serving.WSGIRequestHandler
         to Flask.
 
         :param host: the hostname to listen on. Set this to ``'0.0.0.0'`` to
@@ -131,29 +170,31 @@ class Eve(Flask, Events):
         super(Eve, self).run(host, port, debug, **options)
 
     def load_config(self):
-        """API settings are loaded from standard python modules. First from
+        """ API settings are loaded from standard python modules. First from
         `settings.py`(or alternative name/path passed as an argument) and
         then, when defined, from the file specified in the
         `EVE_SETTINGS` environment variable.
 
         Since we are a Flask subclass, any configuration value supported by
         Flask itself is available (besides Eve's proper settings).
+
+        .. versionchanged:: 0.2
+           Allow use of a dict object as settings.
         """
 
         # load defaults
         self.config.from_object('eve.default_settings')
 
-        # overwrite the defaults with custom user settings:
-
-        # TODO perhaps we should support non-existing settings file, in order
-        # to allow for envvar_only scenarios. However, we should probably
-        # issue a warning
-        if os.path.isabs(self.settings):
-            pyfile = self.settings
+        # overwrite the defaults with custom user settings
+        if isinstance(self.settings, dict):
+            self.config.update(self.settings)
         else:
-            abspath = os.path.abspath(os.path.dirname(sys.argv[0]))
-            pyfile = os.path.join(abspath, self.settings)
-        self.config.from_pyfile(pyfile)
+            if os.path.isabs(self.settings):
+                pyfile = self.settings
+            else:
+                abspath = os.path.abspath(os.path.dirname(sys.argv[0]))
+                pyfile = os.path.join(abspath, self.settings)
+            self.config.from_pyfile(pyfile)
 
         #overwrite settings with custom environment variable
         envvar = 'EVE_SETTINGS'
@@ -177,6 +218,10 @@ class Eve(Flask, Events):
         """ Makes sure that REST methods expressed in the configuration
         settings are supported.
 
+        .. versionchanged:: 0.2.0
+           Default supported methods are now class-level attributes.
+           Resource validation delegated to _validate_resource_settings().
+
         .. versionchanged:: 0.1.0
         Support for PUT method.
 
@@ -186,42 +231,47 @@ class Eve(Flask, Events):
         .. versionchanged:: 0.0.2
             Support for DELETE resource method.
         """
-
-        supported_resource_methods = ['GET', 'POST', 'DELETE']
-        supported_item_methods = ['GET', 'PATCH', 'DELETE', 'PUT']
-
         # make sure that global resource methods are supported.
-        self.validate_methods(supported_resource_methods,
+        self.validate_methods(self.supported_resource_methods,
                               self.config.get('RESOURCE_METHODS'),
                               'resource')
 
         # make sure that global item methods are supported.
-        self.validate_methods(supported_item_methods,
+        self.validate_methods(self.supported_item_methods,
                               self.config.get('ITEM_METHODS'),
                               'item')
 
         # make sure that individual resource/item methods are supported.
         for resource, settings in self.config['DOMAIN'].items():
-            self.validate_methods(supported_resource_methods,
-                                  settings['resource_methods'],
-                                  '[%s] resource ' % resource)
-            self.validate_methods(supported_item_methods,
-                                  settings['item_methods'],
-                                  '[%s] item ' % resource)
+            self._validate_resource_settings(resource, settings)
 
-            # while a resource schema is optional for read-only access,
-            # it is mandatory for write-access to resource/items.
-            if 'POST' in settings['resource_methods'] or \
-               'PATCH' in settings['item_methods']:
-                if len(settings['schema']) == 0:
-                    raise ConfigException('A resource schema must be provided '
-                                          'when POST or PATCH methods are '
-                                          'allowed for a resource [%s].' %
-                                          resource)
+    def _validate_resource_settings(self, resource, settings):
+        """ Validates one resource in configuration settings.
 
-            self.validate_roles('allowed_roles', settings, resource)
-            self.validate_roles('allowed_item_roles', settings, resource)
-            self.validate_schema(resource, settings['schema'])
+        :param resource: name of the resource which settings refer to.
+        :param settings: settings of resource to be validated.
+
+        .. versionadded:: 0.2
+        """
+        self.validate_methods(self.supported_resource_methods,
+                              settings['resource_methods'],
+                              '[%s] resource ' % resource)
+        self.validate_methods(self.supported_item_methods,
+                              settings['item_methods'],
+                              '[%s] item ' % resource)
+
+        # while a resource schema is optional for read-only access,
+        # it is mandatory for write-access to resource/items.
+        if 'POST' in settings['resource_methods'] or \
+           'PATCH' in settings['item_methods']:
+            if len(settings['schema']) == 0:
+                raise ConfigException('A resource schema must be provided '
+                                      'when POST or PATCH methods are allowed'
+                                      'for a resource [%s].' % resource)
+
+        self.validate_roles('allowed_roles', settings, resource)
+        self.validate_roles('allowed_item_roles', settings, resource)
+        self.validate_schema(resource, settings['schema'])
 
     def validate_roles(self, directive, candidate, resource):
         """ Validates that user role directives are syntactically and formally
@@ -262,6 +312,9 @@ class Eve(Flask, Events):
         :param resource: resource name.
         :param schema: schema definition for the resource.
 
+        .. versionchanged:: 0.2
+           Allow ID_FIELD in resource schema if not of 'objectid' type.
+
         .. versionchanged:: 0.1.1
            'collection' setting renamed to 'resource' (data_relation).
            Fix order of string arguments in exception message.
@@ -280,7 +333,8 @@ class Eve(Flask, Events):
             offenders.append(eve.DATE_CREATED)
         if eve.LAST_UPDATED in schema:
             offenders.append(eve.LAST_UPDATED)
-        if eve.ID_FIELD in schema:
+        if eve.ID_FIELD in schema and \
+           schema[eve.ID_FIELD]['type'] == 'objectid':
             offenders.append(eve.ID_FIELD)
         if offenders:
             raise SchemaException('field(s) "%s" not allowed in "%s" schema '
@@ -307,6 +361,10 @@ class Eve(Flask, Events):
         """ When not provided, fills individual resource settings with default
         or global configuration settings.
 
+        .. versionchanged:: 0.2
+           Setting of actual resource defaults is delegated to
+           _set_resource_defaults().
+
         .. versionchanged:: 0.1.1
            'default' values that could be assimilated to None (0, None, "")
            would be ignored.
@@ -351,74 +409,88 @@ class Eve(Flask, Events):
         """
 
         for resource, settings in self.config['DOMAIN'].items():
-            settings.setdefault('url', resource)
-            settings.setdefault('resource_methods',
-                                self.config['RESOURCE_METHODS'])
-            settings.setdefault('public_methods',
-                                self.config['PUBLIC_METHODS'])
-            settings.setdefault('allowed_roles', self.config['ALLOWED_ROLES'])
-            settings.setdefault('cache_control', self.config['CACHE_CONTROL'])
-            settings.setdefault('cache_expires', self.config['CACHE_EXPIRES'])
+            self._set_resource_defaults(resource, settings)
 
-            settings.setdefault('item_lookup_field',
-                                self.config['ITEM_LOOKUP_FIELD'])
-            settings.setdefault('item_url', self.config['ITEM_URL'])
-            settings.setdefault('item_title',
-                                resource.rstrip('s').capitalize())
-            settings.setdefault('item_lookup', self.config['ITEM_LOOKUP'])
-            settings.setdefault('public_item_methods',
-                                self.config['PUBLIC_ITEM_METHODS'])
-            settings.setdefault('allowed_item_roles',
-                                self.config['ALLOWED_ITEM_ROLES'])
-            settings.setdefault('allowed_filters',
-                                self.config['ALLOWED_FILTERS'])
-            settings.setdefault('sorting', self.config['SORTING'])
-            settings.setdefault('embedding', self.config['EMBEDDING'])
-            settings.setdefault('pagination', self.config['PAGINATION'])
-            settings.setdefault('projection', self.config['PROJECTION'])
-            # TODO make sure that this we really need the test below
-            if settings['item_lookup']:
-                item_methods = self.config['ITEM_METHODS']
-            else:
-                item_methods = eve.ITEM_METHODS
-            settings.setdefault('item_methods', item_methods)
-            settings.setdefault('auth_field',
-                                self.config['AUTH_FIELD'])
-            settings.setdefault('allow_unknown', self.config['ALLOW_UNKNOWN'])
-            settings.setdefault('extra_response_fields',
-                                self.config['EXTRA_RESPONSE_FIELDS'])
-            settings.setdefault('mongo_write_concern',
-                                self.config['MONGO_WRITE_CONCERN'])
-            settings.setdefault('hateoas',
-                                self.config['HATEOAS'])
+    def _set_resource_defaults(self, resource, settings):
+        """ Low-level method which sets default values for one resource.
 
-            # empty schemas are allowed for read-only access to resources
-            schema = settings.setdefault('schema', {})
-            self.set_schema_defaults(schema)
+        .. versionchanged:: 0.2
+           'resource_title',
+           'default_sort',
+           'embedded_fields'.
+           Support for endpoint-level authenticatoin classes.
+        """
+        settings.setdefault('url', resource)
+        settings.setdefault('resource_methods',
+                            self.config['RESOURCE_METHODS'])
+        settings.setdefault('public_methods',
+                            self.config['PUBLIC_METHODS'])
+        settings.setdefault('allowed_roles', self.config['ALLOWED_ROLES'])
+        settings.setdefault('cache_control', self.config['CACHE_CONTROL'])
+        settings.setdefault('cache_expires', self.config['CACHE_EXPIRES'])
 
-            datasource = {}
-            settings.setdefault('datasource', datasource)
-            settings['datasource'].setdefault('source', resource)
-            settings['datasource'].setdefault('filter', None)
+        settings.setdefault('item_lookup_field',
+                            self.config['ITEM_LOOKUP_FIELD'])
+        settings.setdefault('item_url', self.config['ITEM_URL'])
+        settings.setdefault('resource_title', settings['url'])
+        settings.setdefault('item_title',
+                            resource.rstrip('s').capitalize())
+        settings.setdefault('item_lookup', self.config['ITEM_LOOKUP'])
+        settings.setdefault('public_item_methods',
+                            self.config['PUBLIC_ITEM_METHODS'])
+        settings.setdefault('allowed_item_roles',
+                            self.config['ALLOWED_ITEM_ROLES'])
+        settings.setdefault('allowed_filters',
+                            self.config['ALLOWED_FILTERS'])
+        settings.setdefault('sorting', self.config['SORTING'])
+        settings.setdefault('embedding', self.config['EMBEDDING'])
+        settings.setdefault('embedded_fields', [])
+        settings.setdefault('pagination', self.config['PAGINATION'])
+        settings.setdefault('projection', self.config['PROJECTION'])
+        # TODO make sure that this we really need the test below
+        if settings['item_lookup']:
+            item_methods = self.config['ITEM_METHODS']
+        else:
+            item_methods = eve.ITEM_METHODS
+        settings.setdefault('item_methods', item_methods)
+        settings.setdefault('auth_field',
+                            self.config['AUTH_FIELD'])
+        settings.setdefault('allow_unknown', self.config['ALLOW_UNKNOWN'])
+        settings.setdefault('extra_response_fields',
+                            self.config['EXTRA_RESPONSE_FIELDS'])
+        settings.setdefault('mongo_write_concern',
+                            self.config['MONGO_WRITE_CONCERN'])
+        settings.setdefault('hateoas',
+                            self.config['HATEOAS'])
+        settings.setdefault('authentication', self.auth if self.auth else None)
+        # empty schemas are allowed for read-only access to resources
+        schema = settings.setdefault('schema', {})
+        self.set_schema_defaults(schema)
 
-            # enable retrieval of actual schema fields only. Eventual db
-            # fields not included in the schema won't be returned.
-            default_projection = {}
-            default_projection.update(dict((field, 1) for (field) in schema))
-            projection = settings['datasource'].setdefault('projection',
-                                                           default_projection)
-            # despite projection, automatic fields are always included.
-            projection[self.config['ID_FIELD']] = 1
-            projection[self.config['LAST_UPDATED']] = 1
-            projection[self.config['DATE_CREATED']] = 1
+        datasource = {}
+        settings.setdefault('datasource', datasource)
+        settings['datasource'].setdefault('source', resource)
+        settings['datasource'].setdefault('filter', None)
+        settings['datasource'].setdefault('default_sort', None)
 
-            # 'defaults' helper set contains the names of fields with
-            # default values in their schema definition.
+        # enable retrieval of actual schema fields only. Eventual db
+        # fields not included in the schema won't be returned.
+        default_projection = {}
+        default_projection.update(dict((field, 1) for (field) in schema))
+        projection = settings['datasource'].setdefault('projection',
+                                                       default_projection)
+        # despite projection, automatic fields are always included.
+        projection[self.config['ID_FIELD']] = 1
+        projection[self.config['LAST_UPDATED']] = 1
+        projection[self.config['DATE_CREATED']] = 1
 
-            # TODO support default values for embedded documents.
-            settings['defaults'] = \
-                set(field for field, definition in schema.items()
-                    if 'default' in definition)
+        # 'defaults' helper set contains the names of fields with
+        # default values in their schema definition.
+
+        # TODO support default values for embedded documents.
+        settings['defaults'] = \
+            set(field for field, definition in schema.items()
+                if 'default' in definition)
 
     def set_schema_defaults(self, schema):
         """ When not provided, fills individual schema settings with default
@@ -440,10 +512,67 @@ class Eve(Flask, Events):
         for data_relation in list(extract_key_values('data_relation', schema)):
             data_relation.setdefault('field', self.config['ID_FIELD'])
 
+    @property
+    def api_prefix(self):
+        """ Prefix to API endpoints.
+
+        .. versionadded:: 0.2
+        """
+        return api_prefix(self.config['URL_PREFIX'],
+                          self.config['API_VERSION'])
+
+    def _add_resource_url_rules(self, resource, settings):
+        """ Builds the API url map for one resource. Methods are enabled for
+        each mapped endpoint, as configured in the settings.
+
+        .. versionadded:: 0.2
+        """
+        url = '%s/%s' % (self.api_prefix, settings['url'])
+        self.config['URLS'][resource] = settings['url']
+        self.config['SOURCES'][resource] = settings['datasource']
+
+        # resource endpoint
+        endpoint = resource + "|resource"
+        self.add_url_rule(url, endpoint, view_func=collections_endpoint,
+                          methods=settings['resource_methods'] + ['OPTIONS'])
+
+        # item endpoint
+        if settings['item_lookup']:
+            item_url = '%s/<%s:%s>' % (url, settings['item_url'],
+                                       settings['item_lookup_field'])
+
+            endpoint = resource + "|item_lookup"
+            self.add_url_rule(item_url, endpoint,
+                              view_func=item_endpoint,
+                              methods=settings['item_methods'] + ['OPTIONS'])
+            if 'PATCH' in settings['item_methods']:
+                # support for POST with X-HTTM-Method-Override header for
+                # clients not supporting PATCH. Also see item_endpoint() in
+                # endpoints.py
+                endpoint = resource + "|post_override"
+                self.add_url_rule(item_url, endpoint, view_func=item_endpoint,
+                                  methods=['POST'])
+
+            # also enable an alternative lookup/endpoint if allowed
+            lookup = settings.get('additional_lookup')
+            if lookup:
+                l_type = settings['schema'][lookup['field']]['type']
+                if l_type == 'integer':
+                    item_url = '%s/<int:%s>' % (url, lookup['field'])
+                else:
+                    item_url = '%s/<%s:%s>' % (url, lookup['url'],
+                                               lookup['field'])
+                endpoint = resource + "|additional_lookup"
+                self.add_url_rule(item_url, endpoint, view_func=item_endpoint,
+                                  methods=['GET', 'OPTIONS'])
+
     def _add_url_rules(self):
         """ Builds the API url map. Methods are enabled for each mapped
         endpoint, as configured in the settings.
 
+        .. versionchanged:: 0.2
+           Delegate adding of resource rules to _add_resource_rules().
+
         .. versionchanged:: 0.1.1
            Simplified URL rules. Not using regexes anymore to return the
            endpoint URL to the endpoint function. This allows for nested
@@ -464,12 +593,8 @@ class Eve(Flask, Events):
            Support for API_VERSION as an endpoint prefix.
         """
         # helpers
-        resources = {}     # maps urls to resources (DOMAIN keys)
-        urls = {}          # maps resources to urls
-        datasources = {}   # maps resources to their datasources
-
-        prefix = api_prefix(self.config['URL_PREFIX'],
-                            self.config['API_VERSION'])
+        self.config['URLS'] = {}       # maps resources to urls
+        self.config['SOURCES'] = {}    # maps resources to their datasources
 
         # we choose not to care about trailing slashes at all.
         # Both '/resource/' and '/resource' will work, same with
@@ -477,47 +602,26 @@ class Eve(Flask, Events):
         self.url_map.strict_slashes = False
 
         # home page (API entry point)
-        self.add_url_rule('%s/' % prefix, 'home', view_func=home_endpoint,
-                          methods=['GET', 'OPTIONS'])
+        self.add_url_rule('%s/' % self.api_prefix, 'home',
+                          view_func=home_endpoint, methods=['GET', 'OPTIONS'])
 
         for resource, settings in self.config['DOMAIN'].items():
-            url = '%s/%s' % (prefix, settings['url'])
-            resources[url] = resource
-            urls[resource] = settings['url']
-            datasources[resource] = settings['datasource']
+            self._add_resource_url_rules(resource, settings)
 
-            # resource endpoint
-            self.add_url_rule(url, view_func=collections_endpoint,
-                              methods=settings['resource_methods'] +
-                              ['OPTIONS'])
+    def register_resource(self, resource, settings):
+        """ Registers new resource to the domain.
 
-            # item endpoint
-            if settings['item_lookup']:
-                item_url = '%s/<regex("%s"):%s>' % \
-                    (url, settings['item_url'], settings['item_lookup_field'])
+        Under the hood this validates given settings, updates default values
+        and adds necessary URL routes (builds api url map).
 
-                self.add_url_rule(item_url, view_func=item_endpoint,
-                                  methods=settings['item_methods']
-                                  + ['OPTIONS'])
-                if 'PATCH' in settings['item_methods']:
-                    # support for POST with X-HTTM-Method-Override header
-                    # for clients not supporting PATCH. Also see
-                    # item_endpoint() in endpoints.py
-                    self.add_url_rule(item_url, view_func=item_endpoint,
-                                      methods=['POST'])
+        If there exists some resource with given name, it is overwritten.
 
-                # also enable an alternative lookup/endpoint if allowed
-                lookup = settings.get('additional_lookup')
-                if lookup:
-                    l_type = settings['schema'][lookup['field']]['type']
-                    if l_type == 'integer':
-                        item_url = '%s/<int:%s>' % (url, lookup['field'])
-                    else:
-                        item_url = '%s/<regex("%s"):%s>' % (url,
-                                                            lookup['url'],
-                                                            lookup['field'])
-                    self.add_url_rule(item_url, view_func=item_endpoint,
-                                      methods=['GET', 'OPTIONS'])
-        self.config['RESOURCES'] = resources
-        self.config['URLS'] = urls
-        self.config['SOURCES'] = datasources
+        :param resource: resource name.
+        :param settings: settings for given resource.
+
+        .. versionadded:: 0.2
+        """
+        self.config['DOMAIN'][resource] = settings
+        self._set_resource_defaults(resource, settings)
+        self._validate_resource_settings(resource, settings)
+        self._add_resource_url_rules(resource, settings)
