@@ -11,11 +11,29 @@
 """
 from eve.utils import config, debug_error_message
 from flask import request, abort
+import simplejson as json
+from eve.utils import date_to_str
+import datetime
+
+
+class BaseJSONEncoder(json.JSONEncoder):
+    """ Proprietary JSONEconder subclass used by the json render function.
+    This is needed to address the encoding of special values.
+    """
+    def default(self, obj):
+        if isinstance(obj, datetime.datetime):
+            # convert any datetime to RFC 1123 format
+            return date_to_str(obj)
+        elif isinstance(obj, (datetime.time, datetime.date)):
+            # should not happen since the only supported date-like format
+            # supported at dmain schema level is 'datetime' .
+            return obj.isoformat()
+        return json.JSONEncoder.default(self, obj)
 
 
 class ConnectionException(Exception):
-    """Raised when DataLayer subclasses cannot find/activate to their
-    database connection
+    """ Raised when DataLayer subclasses cannot find/activate to their
+    database connection.
 
     :param driver_exception: the original exception raised by the source db
                              driver
@@ -39,6 +57,9 @@ class DataLayer(object):
     Admittedly, this interface is a Mongo rip-off. See the io.mongo
     package for an implementation example.
 
+    .. versionchanged:: 0.2
+       Allow subclasses to provide their own specialized json encoder.
+
     .. versionchanged:: 0.1.1
        'serializers' dictionary added.
 
@@ -60,9 +81,18 @@ class DataLayer(object):
     # serializers = {'objectid': ObjectId, 'datetime': serialize_date}
     serializers = {}
 
+    # json.JSONEncoder subclass for serializing data to json.
+    # Subclasses should provide their own specialized encoder (see
+    # eve.io.mongo.MongoJSONEncoder).
+    json_encoder_class = BaseJSONEncoder
+
     def __init__(self, app):
         """ Implements the Flask extension pattern.
+
+        .. versionchanged:: 0.2
+           Explicit initialize self.driver to None.
         """
+        self.driver = None
         if app is not None:
             self.app = app
             self.init_app(self.app)
@@ -95,7 +125,7 @@ class DataLayer(object):
         raise NotImplementedError
 
     def find_one(self, resource, **lookup):
-        """Retrieves a single document/record. Consumed when a request hits an
+        """ Retrieves a single document/record. Consumed when a request hits an
         item endpoint (`/people/id/`).
 
         :param resource: resource being accessed. You should then use the
@@ -110,10 +140,10 @@ class DataLayer(object):
         raise NotImplementedError
 
     def find_list_of_ids(self, resource, ids, client_projection=None):
-        """Retrieves a list of documents based on a list of primary keys
+        """ Retrieves a list of documents based on a list of primary keys
         The primary key is the field defined in `ID_FIELD`.
         This is a separate function to allow us to use per-database
-        optimizations for this type of query
+        optimizations for this type of query.
 
         :param resource: resource name.
         :param ids: a list of ids corresponding to the documents
@@ -127,7 +157,7 @@ class DataLayer(object):
         raise NotImplementedError
 
     def insert(self, resource, doc_or_docs):
-        """Inserts a document into a resource collection/table.
+        """ Inserts a document into a resource collection/table.
 
         :param resource: resource being accessed. You should then use
                          the ``_datasource`` helper function to retrieve both
@@ -142,7 +172,7 @@ class DataLayer(object):
         raise NotImplementedError
 
     def update(self, resource, id_, updates):
-        """Updates a collection/table document/row.
+        """ Updates a collection/table document/row.
         :param resource: resource being accessed. You should then use
                          the ``_datasource`` helper function to retrieve
                          the actual datasource name.
@@ -150,11 +180,10 @@ class DataLayer(object):
         :param updates: json updates to be performed on the database document
                         (or row).
         """
-
         raise NotImplementedError
 
     def replace(self, resource, id_, document):
-        """Replaces a collection/table document/row.
+        """ Replaces a collection/table document/row.
         :param resource: resource being accessed. You should then use
                          the ``_datasource`` helper function to retrieve
                          the actual datasource name.
@@ -163,11 +192,10 @@ class DataLayer(object):
 
         .. versionadded:: 0.1.0
         """
-
         raise NotImplementedError
 
     def remove(self, resource, id_=None):
-        """Removes a document/row or an entire set of documents/rows from a
+        """ Removes a document/row or an entire set of documents/rows from a
         database collection/table.
 
         :param resource: resource being accessed. You should then use
@@ -181,8 +209,7 @@ class DataLayer(object):
         raise NotImplementedError
 
     def combine_queries(self, query_a, query_b):
-        """
-        Takes two db queries and applies db-specific syntax to produce
+        """ Takes two db queries and applies db-specific syntax to produce
         the intersection.
 
         .. versionadded: 0.1.0
@@ -191,8 +218,7 @@ class DataLayer(object):
         raise NotImplementedError
 
     def get_value_from_query(self, query, field_name):
-        """
-        Parses the given potentially-complex query and returns the value
+        """ Parses the given potentially-complex query and returns the value
         being assigned to the field given in `field_name`.
 
         This mainly exists to deal with more complicated compound queries
@@ -204,7 +230,7 @@ class DataLayer(object):
 
     def query_contains_field(self, query, field_name):
         """ For the specified field name, does the query contain it?
-        Used know whether we need to parse a compound query
+        Used know whether we need to parse a compound query.
 
         .. versionadded: 0.1.0
            Support for parsing values embedded in compound db queries
@@ -212,20 +238,30 @@ class DataLayer(object):
         raise NotImplementedError
 
     def _datasource(self, resource):
-        """Returns a tuple with the actual name of the database
+        """ Returns a tuple with the actual name of the database
         collection/table, base query and projection for the resource being
         accessed.
 
         :param resource: resource being accessed.
-        """
 
+        .. versionchanged:: 0.2
+           Support for 'default_sort'.
+        """
         return (config.SOURCES[resource]['source'],
                 config.SOURCES[resource]['filter'],
-                config.SOURCES[resource]['projection'])
+                config.SOURCES[resource]['projection'],
+                config.SOURCES[resource]['default_sort'],
+                )
 
-    def _datasource_ex(self, resource, query=None, client_projection=None):
+    def _datasource_ex(self, resource, query=None, client_projection=None,
+                       client_sort=None):
         """ Returns both db collection and exact query (base filter included)
-        to which an API resource refers to
+        to which an API resource refers to.
+
+        .. versionchanged:: 0.2
+           Difference between resource and item endpoints is now determined
+           by the presence of a '|' in request.endpoint.
+           Support for 'default_sort'.
 
         .. versionchanged:: 0.1.1
            auth.request_auth_value is now used to store the auth_field value.
@@ -249,7 +285,17 @@ class DataLayer(object):
         .. versionadded:: 0.0.4
         """
 
-        datasource, filter_, projection_ = self._datasource(resource)
+        datasource, filter_, projection_, sort_ = self._datasource(resource)
+
+        if client_sort:
+            sort = client_sort
+        else:
+            # default sort is activated only if 'sorting' is enabled for the
+            # resource.
+            # TODO Consider raising a validation error on startup instead?
+            sort = sort_ if sort_ and config.DOMAIN[resource]['sorting'] else \
+                None
+
         if filter_:
             if query:
                 # Can't just dump one set of query operators into another
@@ -277,25 +323,28 @@ class DataLayer(object):
         # If the current HTTP method is in `public_methods` or
         # `public_item_methods`, skip the `auth_field` check
 
-        if request.endpoint == 'collections_endpoint':
-            # We need to check against `public_methods`
+        if '|resource' in request.endpoint:
+            # We are on a resource endpoint and need to check against
+            # `public_methods`
             public_method_list_to_check = 'public_methods'
         else:
-            # We need to check against `public_item_methods`
+            # We are on an item endpoint and need to check against
+            # `public_item_methods`
             public_method_list_to_check = 'public_item_methods'
 
         # Is the HTTP method not public?
         resource_dict = config.DOMAIN[resource]
+        auth = resource_dict['authentication']
+        request_auth_value = auth.request_auth_value if auth else None
         if request.method not in resource_dict[public_method_list_to_check]:
             # We need to run the 'user-restricted resource access' check
             auth_field = resource_dict.get('auth_field', None)
-            if auth_field and request.authorization and self.app.auth \
+            if auth_field and request.authorization and request_auth_value \
                     and query is not None:
                 # If the auth_field *replaces* a field in the query,
                 # and the values are /different/, deny the request
                 # This prevents the auth_field condition from
                 # overwriting the query (issue #77)
-                request_auth_value = self.app.auth.request_auth_value
                 auth_field_in_query = \
                     self.app.data.query_contains_field(query, auth_field)
                 if auth_field_in_query and \
@@ -315,4 +364,4 @@ class DataLayer(object):
                     query = self.app.data.combine_queries(
                         query, {auth_field: request_auth_value}
                     )
-        return datasource, query, fields
+        return datasource, query, fields, sort
