@@ -17,7 +17,7 @@ from bson import ObjectId       # noqa
 
 
 def parse(expression):
-    """Given a python-like conditional statement, returns the equivalent
+    """ Given a python-like conditional statement, returns the equivalent
     mongo-like query expression. Conditional and boolean operators (==, <=, >=,
     !=, >, <) along with a couple function calls (ObjectId(), datetime()) are
     supported.
@@ -32,7 +32,7 @@ class ParseError(ValueError):
 
 
 class MongoVisitor(ast.NodeVisitor):
-    """Implements the python-to-mongo parser. Only Python conditional
+    """ Implements the python-to-mongo parser. Only Python conditional
     statements are supported, however nested, combined with most common compare
     and boolean operators (And and Or).
 
@@ -78,7 +78,6 @@ class MongoVisitor(ast.NodeVisitor):
     def visit_Compare(self, node):
         """ Compare operator handler.
         """
-
         self.visit(node.left)
         left = self.current_value
 
@@ -114,7 +113,7 @@ class MongoVisitor(ast.NodeVisitor):
 
     def visit_Call(self, node):
         """ A couple function calls are supported: bson's ObjectId() and
-        datetime()
+        datetime().
         """
         if isinstance(node.func, ast.Name):
             expr = None
@@ -129,19 +128,22 @@ class MongoVisitor(ast.NodeVisitor):
                 self.current_value = eval(node.func.id + expr)
 
     def visit_Attribute(self, node):
-        """ Attribute handler ('Contact.Id')
+        """ Attribute handler ('Contact.Id').
         """
         self.visit(node.value)
         self.current_value += "." + node.attr
 
     def visit_Name(self, node):
-        """ Names """
+        """ Names handler.
+        """
         self.current_value = node.id
 
     def visit_Num(self, node):
-        """ Numbers """
+        """ Numbers handler.
+        """
         self.current_value = node.n
 
     def visit_Str(self, node):
-        """ Strings """
+        """ Strings handler.
+        """
         self.current_value = node.s
