@@ -36,32 +36,32 @@ class Validator(Validator):
     """
     def __init__(self, schema, resource=None):
         self.resource = resource
-        self.object_id = None
+        self._id = None
         super(Validator, self).__init__(schema, transparent_schema_rules=True)
         if resource:
             self.allow_unknown = config.DOMAIN[resource]['allow_unknown']
 
-    def validate_update(self, document, object_id):
+    def validate_update(self, document, _id):
         """ Validate method to be invoked when performing an update, not an
         insert.
 
         :param document: the document to be validated.
-        :param object_id: the unique id of the document.
+        :param _id: the unique id of the document.
         """
-        self.object_id = object_id
+        self._id = _id
         return super(Validator, self).validate_update(document)
 
-    def validate_replace(self, document, object_id):
+    def validate_replace(self, document, _id):
         """ Validation method to be invoked when performing a document
         replacement. This differs from :func:`validation_update` since in this
         case we want to perform a full :func:`validate` (the new document is to
         be considered a new insertion and required fields needs validation).
-        However, like with validate_update, we also want the current object_id
+        However, like with validate_update, we also want the current _id
         not to be checked when validationg 'unique' values.
 
         .. versionadded:: 0.1.0
         """
-        self.object_id = object_id
+        self._id = _id
         return super(Validator, self).validate(document)
 
     def _validate_unique(self, unique, field, value):
@@ -71,11 +71,18 @@ class Validator(Validator):
                        unique or not.
         :param field: field name.
         :param value: field value.
+
+        .. versionchanged:: 0.2
+           Handle the case in which ID_FIELD is not of ObjectId type.
         """
         if unique:
             query = {field: value}
-            if self.object_id:
-                query[config.ID_FIELD] = {'$ne': ObjectId(self.object_id)}
+            if self._id:
+                try:
+                    query[config.ID_FIELD] = {'$ne': ObjectId(self._id)}
+                except:
+                    query[config.ID_FIELD] = {'$ne': self._id}
+
             if app.data.find_one(self.resource, **query):
                 self._error("value '%s' for field '%s' not unique" %
                             (value, field))
@@ -98,10 +105,10 @@ class Validator(Validator):
         """
         query = {data_relation['field']: value}
         if not app.data.find_one(data_relation['resource'], **query):
-                self._error("value '%s' for field '%s' must exist in "
-                            "resource '%s', field '%s'" %
-                            (value, field, data_relation['resource'],
-                             data_relation['field']))
+            self._error("value '%s' for field '%s' must exist in "
+                        "resource '%s', field '%s'" %
+                        (value, field, data_relation['resource'],
+                         data_relation['field']))
 
     def _validate_type_objectid(self, field, value):
         """ Enables validation for `objectid` schema attribute.
@@ -112,7 +119,7 @@ class Validator(Validator):
         :param value: field value.
 
         .. versionchanged:: 0.1.1
-           regex check replaced by proper type check.
+           regex check replaced with proper type check.
         """
         if not isinstance(value, ObjectId):
             self._error("value '%s' for field '%s' cannot be converted to a "
