@@ -17,13 +17,14 @@ from eve.utils import document_etag, document_link, config, debug_error_message
 from eve.auth import requires_auth
 from eve.validation import ValidationError
 from eve.methods.common import get_document, parse, payload as payload_, \
-    ratelimit
+    ratelimit, pre_event
 
 
 @ratelimit()
 @requires_auth('item')
+@pre_event
 def patch(resource, **lookup):
-    """Perform a document patch/update. Updates are first validated against
+    """ Perform a document patch/update. Updates are first validated against
     the resource schema. If validation passes, the document is updated and
     an OK status update is returned. If validation fails, a set of validation
     issues is returned.
@@ -31,6 +32,11 @@ def patch(resource, **lookup):
     :param resource: the name of the resource to which the document belongs.
     :param **lookup: document lookup query.
 
+    .. versionchanged:: 0.2
+       Use the new STATUS setting.
+       Use the new ISSUES setting.
+       Raise 'on_pre_<method>' event.
+
     .. versionchanged:: 0.1.1
        Item-identifier wrapper stripped from both request and response payload.
 
@@ -102,8 +108,8 @@ def patch(resource, **lookup):
             # metadata
             response['etag'] = etag
             if resource_def['hateoas']:
-                response['_links'] = {'self': document_link(resource,
-                                                            object_id)}
+                response[config.LINKS] = {'self': document_link(resource,
+                                                                object_id)}
         else:
             issues.extend(validator.errors)
     except ValidationError as e:
@@ -119,9 +125,9 @@ def patch(resource, **lookup):
         ))
 
     if len(issues):
-        response['issues'] = issues
-        response['status'] = config.STATUS_ERR
+        response[config.ISSUES] = issues
+        response[config.STATUS] = config.STATUS_ERR
     else:
-        response['status'] = config.STATUS_OK
+        response[config.STATUS] = config.STATUS_OK
 
     return response, last_modified, etag, 200
