@@ -9,7 +9,15 @@ from eve.tests import TestBase
 
 
 class ValidBasicAuth(BasicAuth):
+    def __init__(self):
+        self.skip_setting_auth_value = False
+        super(ValidBasicAuth, self).__init__()
+
     def check_auth(self, username, password, allowed_roles, resource, method):
+        # ignoble hack to only set request_auth_value when the test needs it to
+        # be really set
+        if not self.skip_setting_auth_value:
+            self.request_auth_value = 'admin'
         return username == 'admin' and password == 'secret' and  \
             (allowed_roles == ['admin'] if allowed_roles else True)
 
@@ -46,13 +54,13 @@ class TestBasicAuth(TestBase):
                            self.content_type]
         self.invalid_auth = [('Authorization', 'Basic IDontThinkSo'),
                              self.content_type]
-        for resource, schema in self.app.config['DOMAIN'].items():
+        for _, schema in self.app.config['DOMAIN'].items():
             schema['allowed_roles'] = ['admin']
             schema['allowed_item_roles'] = ['admin']
         self.app.set_defaults()
 
     def test_custom_auth(self):
-        self.assertEqual(type(self.app.auth), ValidBasicAuth)
+        self.assertTrue(isinstance(self.app.auth, ValidBasicAuth))
 
     def test_restricted_home_access(self):
         r = self.test_client.get('/')
@@ -137,6 +145,7 @@ class TestBasicAuth(TestBase):
         for resource, settings in domain.items():
             del(settings['public_methods'])
         self.app.set_defaults()
+        del(domain['peopleinvoices'])
         for resource in domain:
             url = self.app.config['URLS'][resource]
             r = self.test_client.get(url)
@@ -150,7 +159,7 @@ class TestBasicAuth(TestBase):
     def test_public_methods_but_locked_resource(self):
         self.app.config['PUBLIC_METHODS'] = ['GET']
         domain = self.app.config['DOMAIN']
-        for resource, settings in domain.items():
+        for _, settings in domain.items():
             del(settings['public_methods'])
         self.app.set_defaults()
         domain[self.known_resource]['public_methods'] = []
@@ -160,7 +169,7 @@ class TestBasicAuth(TestBase):
     def test_public_methods_but_locked_item(self):
         self.app.config['PUBLIC_ITEM_METHODS'] = ['GET']
         domain = self.app.config['DOMAIN']
-        for resource, settings in domain.items():
+        for _, settings in domain.items():
             del(settings['public_item_methods'])
         self.app.set_defaults()
         domain[self.known_resource]['public_item_methods'] = []
@@ -169,7 +178,7 @@ class TestBasicAuth(TestBase):
 
     def test_public_methods_item(self):
         self.app.config['PUBLIC_ITEM_METHODS'] = ['GET']
-        for resource, settings in self.app.config['DOMAIN'].items():
+        for _, settings in self.app.config['DOMAIN'].items():
             del(settings['public_item_methods'])
         self.app.set_defaults()
         # we're happy with testing just one client endpoint, but for sake of
@@ -204,7 +213,7 @@ class TestTokenAuth(TestBasicAuth):
                            self.content_type]
 
     def test_custom_auth(self):
-        self.assertEqual(type(self.app.auth), ValidTokenAuth)
+        self.assertTrue(isinstance(self.app.auth, ValidTokenAuth))
 
 
 class TestHMACAuth(TestBasicAuth):
@@ -216,7 +225,7 @@ class TestHMACAuth(TestBasicAuth):
                            self.content_type]
 
     def test_custom_auth(self):
-        self.assertEqual(type(self.app.auth), ValidHMACAuth)
+        self.assertTrue(isinstance(self.app.auth, ValidHMACAuth))
 
     def test_bad_auth_class(self):
         self.app = Eve(settings=self.settings_file, auth=BadHMACAuth)
@@ -233,27 +242,28 @@ class TestHMACAuth(TestBasicAuth):
 class TestUserRestrictedAccess(TestBase):
     def setUp(self):
         super(TestUserRestrictedAccess, self).setUp()
+
         self.app = Eve(settings=self.settings_file, auth=ValidBasicAuth)
-        # remove the datasource filter to make the whole collection available
-        # to a GET request.
-        self.resource = self.app.config['DOMAIN'][self.known_resource]
-        del self.resource['datasource']['filter']
-        self.app.set_defaults()
-        self.app._add_url_rules()
+
+        # using this endpoint since it is a copy of 'contacts' with
+        # no filter on the datasource
+        self.url = 'restricted'
+        self.resource = self.app.config['DOMAIN'][self.url]
         self.test_client = self.app.test_client()
+
         self.valid_auth = [('Authorization', 'Basic YWRtaW46c2VjcmV0')]
         self.invalid_auth = [('Authorization', 'Basic IDontThinkSo')]
         self.field_name = 'auth_field'
         self.data = json.dumps({"ref": "0123456789123456789012345"})
-        for resource, settings in self.app.config['DOMAIN'].items():
+
+        for _, settings in self.app.config['DOMAIN'].items():
             settings[self.field_name] = 'username'
+
         self.resource['public_methods'] = []
-        self.app.auth.request_auth_value = 'admin'
 
     def test_get(self):
         data, status = self.parse_response(
-            self.test_client.get(self.known_resource_url,
-                                 headers=self.valid_auth))
+            self.test_client.get(self.url, headers=self.valid_auth))
         self.assert200(status)
         # no data has been saved by user 'admin' yet,
         # so assert we get an empty result set back.
@@ -267,7 +277,7 @@ class TestUserRestrictedAccess(TestBase):
 
         # Verify that we can retrieve it
         data2, status2 = self.parse_response(
-            self.test_client.get(self.known_resource_url,
+            self.test_client.get(self.url,
                                  headers=self.valid_auth))
         self.assert200(status2)
         self.assertEqual(len(data2['_items']), 1)
@@ -280,7 +290,7 @@ class TestUserRestrictedAccess(TestBase):
         This test verifies that the `auth_field` does not overwrite
         a `client_filter` or url param.
         """
-        data, status = self.parse_response(
+        _, status = self.parse_response(
             self.test_client.get(self.user_username_url,
                                  headers=self.valid_auth))
         self.assert401(status)
@@ -292,7 +302,7 @@ class TestUserRestrictedAccess(TestBase):
         self.app.config['DOMAIN']['users'][self.field_name] = \
             self.app.config['ID_FIELD']
 
-        data, status = self.parse_response(
+        _, status = self.parse_response(
             self.test_client.get(self.user_id_url,
                                  headers=self.valid_auth))
         self.assert401(status)
@@ -303,18 +313,19 @@ class TestUserRestrictedAccess(TestBase):
         same
         """
         _id = ObjectId('deadbeefdeadbeefdeadbeef')
-        self.app.auth.request_auth_value = _id
+        resource_def = self.app.config['DOMAIN']['users']
+        resource_def['authentication'].skip_setting_auth_value = True
+        resource_def['authentication'].request_auth_value = _id
 
         # set auth_field to `_id`
-        self.app.config['DOMAIN']['users'][self.field_name] = \
-            self.app.config['ID_FIELD']
+        resource_def[self.field_name] = '_id'
 
         # Retrieving a /different user/ by id returns 401
         user_url = '/users/'
         filter_by_id = 'where=_id==ObjectId("%s")'
         filter_query = filter_by_id % self.user_id
 
-        data, status = self.parse_response(
+        _, status = self.parse_response(
             self.test_client.get('%s?%s' % (user_url, filter_query),
                                  headers=self.valid_auth))
         self.assert401(status)
@@ -340,7 +351,7 @@ class TestUserRestrictedAccess(TestBase):
         """
         self.resource['public_methods'].append('GET')
         data, status = self.parse_response(
-            self.test_client.get(self.known_resource_url))      # no auth
+            self.test_client.get(self.url))      # no auth
         self.assert200(status)
         # no data has been saved by user 'admin' yet,
         # but we should get all the other results back
@@ -358,10 +369,10 @@ class TestUserRestrictedAccess(TestBase):
         self.assertEqual(data['_id'], self.item_id)
 
     def test_post(self):
-        response, status = self.post()
+        _, status = self.post()
         self.assert200(status)
         data, status = self.parse_response(
-            self.test_client.get(self.known_resource_url,
+            self.test_client.get(self.url,
                                  headers=self.valid_auth))
         self.assert200(status)
         # len of 1 as there are is only 1 doc saved by user
@@ -371,7 +382,7 @@ class TestUserRestrictedAccess(TestBase):
         new_ref = "9999999999999999999999999"
         changes = json.dumps({"ref": new_ref})
         data, status = self.post()
-        url = '%s/%s' % (self.known_resource_url, data['_id'])
+        url = '%s/%s' % (self.url, data['_id'])
         response = self.test_client.get(url, headers=self.valid_auth)
         etag = response.headers['ETag']
         headers = [('If-Match', etag),
@@ -389,7 +400,7 @@ class TestUserRestrictedAccess(TestBase):
 
     def test_delete(self):
         data, status = self.post()
-        url = '%s/%s' % (self.known_resource_url, data['_id'])
+        url = '%s/%s' % (self.url, data['_id'])
         response = self.test_client.get(url, headers=self.valid_auth)
         etag = response.headers['ETag']
         headers = [('If-Match', etag),
@@ -399,8 +410,46 @@ class TestUserRestrictedAccess(TestBase):
         self.assert200(status)
 
     def post(self):
-        r = self.test_client.post(self.known_resource_url,
+        r = self.test_client.post(self.url,
                                   data=self.data,
                                   headers=self.valid_auth,
                                   content_type='application/json')
         return self.parse_response(r)
+
+
+class TestResourceAuth(TestBase):
+    def test_resource_only_auth(self):
+        # no auth at the API level
+        self.app = Eve(settings=self.settings_file)
+        self.test_client = self.app.test_client()
+        # explicit auth for just one resource
+        self.app.config['DOMAIN']['contacts']['authentication'] = \
+            ValidBasicAuth()
+        self.app.config['DOMAIN']['empty']['authentication'] = ValidTokenAuth()
+        self.app.set_defaults()
+        basic_auth = [('Authorization', 'Basic YWRtaW46c2VjcmV0')]
+        token_auth = [('Authorization', 'Basic dGVzdF90b2tlbjo=')]
+
+        # 'contacts' endpoints are protected
+        r = self.test_client.get(self.known_resource_url)
+        self.assert401(r.status_code)
+        r = self.test_client.get(self.item_id_url)
+        self.assert401(r.status_code)
+        # both with BasicAuth.
+        _, status = self.parse_response(
+            self.test_client.get(self.known_resource_url, headers=basic_auth))
+        self.assert200(status)
+        _, status = self.parse_response(
+            self.test_client.get(self.item_id_url, headers=basic_auth))
+        self.assert200(status)
+
+        # 'empty' resource endpoint is also protected
+        r = self.test_client.get(self.empty_resource_url)
+        self.assert401(r.status_code)
+        # but with TokenAuth
+        r = self.test_client.get(self.empty_resource_url, headers=token_auth)
+        self.assert200(r.status_code)
+
+        # other resources are not protected
+        r = self.test_client.get(self.readonly_resource_url)
+        self.assert200(r.status_code)
