@@ -1,10 +1,12 @@
 from eve.tests import TestBase
+from eve.tests.test_settings import MONGO_DBNAME
+from bson import ObjectId
 
 
 class TestDelete(TestBase):
     def test_unknown_resource(self):
         url = '%s%s/' % (self.unknown_resource_url, self.item_id)
-        r, status = self.delete(url)
+        _, status = self.delete(url)
         self.assert404(status)
 
     def test_delete_from_resource_endpoint(self):
@@ -18,7 +20,7 @@ class TestDelete(TestBase):
     def test_delete_from_resource_endpoint_write_concern(self):
         # should get a 500 since there's no replicaset on the mongod instance
         self.domain['contacts']['mongo_write_concern'] = {'w': 2}
-        r, status = self.delete(self.known_resource_url)
+        _, status = self.delete(self.known_resource_url)
         self.assert500(status)
 
     def test_delete_from_resource_endpoint_different_resource(self):
@@ -38,24 +40,29 @@ class TestDelete(TestBase):
 
     def test_delete_empty_resource(self):
         url = '%s%s/' % (self.empty_resource_url, self.item_id)
-        r, status = self.delete(url)
+        _, status = self.delete(url)
         self.assert404(status)
 
     def test_delete_readonly_resource(self):
-        r, status = self.delete(self.readonly_id_url)
+        _, status = self.delete(self.readonly_id_url)
         self.assert405(status)
 
     def test_delete_unknown_item(self):
         url = '%s%s/' % (self.known_resource_url, self.unknown_item_id)
-        r, status = self.delete(url)
+        _, status = self.delete(url)
         self.assert404(status)
 
     def test_delete_ifmatch_missing(self):
-        r, status = self.delete(self.item_id_url)
+        _, status = self.delete(self.item_id_url)
         self.assert403(status)
 
+    def test_delete_ifmatch_disabled(self):
+        self.app.config['IF_MATCH'] = False
+        _, status = self.delete(self.item_id_url)
+        self.assert200(status)
+
     def test_delete_ifmatch_bad_etag(self):
-        r, status = self.delete(self.item_id_url,
+        _, status = self.delete(self.item_id_url,
                                 headers=[('If-Match', 'not-quite-right')])
         self.assert412(status)
 
@@ -70,7 +77,7 @@ class TestDelete(TestBase):
     def test_delete_write_concern(self):
         # should get a 500 since there's no replicaset on the mongod instance
         self.domain['contacts']['mongo_write_concern'] = {'w': 2}
-        r, status = self.delete(self.item_id_url,
+        _, status = self.delete(self.item_id_url,
                                 headers=[('If-Match', self.item_etag)])
         self.assert500(status)
 
@@ -89,6 +96,28 @@ class TestDelete(TestBase):
         r = self.test_client.post(self.item_id_url, data={}, headers=headers)
         self.assert200(r.status_code)
 
+    def test_delete_subresource(self):
+        _db = self.connection[MONGO_DBNAME]
+
+        # create random contact
+        fake_contact = self.random_contacts(1)
+        fake_contact_id = _db.contacts.insert(fake_contact)[0]
+
+        # update first invoice to reference the new contact
+        _db.invoices.update({'_id': ObjectId(self.invoice_id)},
+                            {'$set': {'person': fake_contact_id}})
+
+        # GET all invoices by new contact
+        response, status = self.get('users/%s/invoices/%s' %
+                                    (fake_contact_id, self.invoice_id))
+        etag = response['etag']
+
+        headers = [('If-Match', etag)]
+        response, status = self.delete('users/%s/invoices/%s' %
+                                       (fake_contact_id, self.invoice_id),
+                                       headers=headers)
+        self.assert200(status)
+
     def delete(self, url, headers=None):
         r = self.test_client.delete(url, headers=headers)
         return self.parse_response(r)
