@@ -14,7 +14,7 @@ ITEM_METHODS = ['GET', 'PATCH', 'DELETE', 'PUT']
 ITEM_CACHE_CONTROL = ''
 ITEM_LOOKUP = True
 ITEM_LOOKUP_FIELD = ID_FIELD
-ITEM_URL = '[a-f0-9]{24}'
+#ITEM_URL = '[a-f0-9]{24}'
 
 contacts = {
     'url': 'arbitraryurl',
@@ -22,7 +22,7 @@ contacts = {
     'cache_expires': 20,
     'item_title': 'contact',
     'additional_lookup': {
-        'url': '[\w]+',   # to be unique field
+        'url': 'regex("[\w]+")',   # to be unique field
         'field': 'ref'
     },
     'datasource': {'filter': {'username': {'$exists': False}}},
@@ -121,6 +121,14 @@ payments = {
 
 empty = copy.deepcopy(invoices)
 
+user_restricted_access = copy.deepcopy(contacts)
+user_restricted_access['url'] = 'restricted'
+user_restricted_access['datasource'] = {'source': 'contacts'}
+
+users_invoices = copy.deepcopy(invoices)
+users_invoices['url'] = 'users/<regex("[a-f0-9]{24}"):person>/invoices'
+users_invoices['datasource'] = {'source': 'invoices'}
+
 DOMAIN = {
     'contacts': contacts,
     'users': users,
@@ -128,4 +136,6 @@ DOMAIN = {
     'invoices': invoices,
     'payments': payments,
     'empty': empty,
+    'restricted': user_restricted_access,
+    'peopleinvoices': users_invoices,
 }
