@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from __future__ import print_function
 
 """
     Custom event notifications
@@ -24,7 +25,7 @@ app = Eve()
 
 @app.before_request
 def before():
-    print 'the request object ready to be processed:', request
+    print('the request object ready to be processed:', request)
 
 
 @app.after_request
@@ -33,7 +34,7 @@ def after(response):
     Your function must take one parameter, a `response_class` object and return
     a new response object or the same (see Flask documentation).
     """
-    print 'and here we have the response object instead:', response
+    print('and here we have the response object instead:', response)
     return response
 
 if __name__ == '__main__':
