@@ -10,29 +10,20 @@ from redis import Redis
 
 redis = Redis()
 
-def test(value):
-    print "inside", value
-    return "$" + value
-
 class TAuth(TokenAuth):
     def check_auth(self, token, allowed_roles, resource, method):
         return token == 'token'
 
 class Auth(BasicAuth):
     def check_auth(self, username, password, allowed_roles, resource, method):
-        if resource is None:
-            print "NONE"
-            return True
-        #print username
-        self.user_id = username
+        self.request_auth_value = "nicola"
         return username == 'token'
         #return username == 'admin' and password == 'secret'
 
 
 class Validator(Validator):
-    def _validate_cin(self, cin, field, value):
-        if cin:
-            pass
+    def _validate_msg_limit(self, msg_limit, field, value):
+        print msg_limit, field, value
 
 def getting_contatti(documents):
     print ("contatti")
@@ -40,6 +31,7 @@ def getting_contatti(documents):
 
 def getting(resource, documents):
     print ("getting %s" % resource)
+    #del(documents[0]['etag'])
     documents[0]['name']="RIFATTO"
     documents[0]['newfield'] = " NEW FIELD YOU FOOL "
 
@@ -57,30 +49,35 @@ def resource_gotit(request, response):
 def resource_post(request, response):
     print ('post_contacts')
     print (request.authorization)
-    print (request.form)
-    print (response)
+    print (request.get_data())
+    print (response.get_data())
 
-def posting(resource, documents):
-    print ("posting to ", resource)
+def posting(documents):
+    #print ("posting to ", resource)
     documents[0]["token"] = "mytoken"
+    print documents
 
 def posting_r(documents):
     print ("posting to resource contacts")
 
 
-app = Eve(validator=Validator, redis=redis)
+app = Eve(auth=Auth, validator=Validator, redis=redis)
 
-app.config['CONV'] = {'string': str}
+#@app.route('/test')
+#def hello_world():
+#    return 'Hello World'
 #app.config['DOMAIN']['contacts']['auth_username_field'] = 'name'
 #app.on_get_contacts += resource_gotit
 #app.on_get += gotit
-#app.on_post_contacts += resource_post
+#app.on_pre_GET += getting
+#app.on_post_POST_contacts += resource_post
 #app.on_posting += posting
 #app.on_posting_contacts += posting_r
-#app.on_getting += getting
+#app.on_fetch_resource += getting
+#app.on_insert_contacts += posting
 #app.on_getting_contacts += getting_contatti
 
 if __name__ == '__main__':
     #app.events.on_getting += pre
-    app.on_GET += gotit
+    #app.on_GET += gotit
     app.run(debug=True)
